package envoy.server.processors;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import envoy.data.Contacts;
import envoy.data.LoginCredentials;
import envoy.data.Message.MessageStatus;
import envoy.data.User;
import envoy.server.ConnectionManager;
import envoy.server.ObjectProcessor;
import envoy.server.data.Message;
import envoy.server.database.PersistenceManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * This {@link ObjectProcessor} handles {@link LoginCredentials}.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>LoginCredentialProcessor.java</strong><br>
 * Created: <strong>30.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-alpha
 */
public class LoginCredentialProcessor implements ObjectProcessor<LoginCredentials> {

	private PersistenceManager	persistenceManager	= PersistenceManager.getPersistenceManager();

	@Override
	public Class<LoginCredentials> getInputClass() { return LoginCredentials.class; }

	@Override
	public void process(LoginCredentials input, long socketId, ObjectWriteProxy writeProxy) throws IOException {
		System.out.println(String.format("Received login credentials %s from socket ID %d", input, socketId));

		// Create user
		envoy.server.data.User user = new envoy.server.data.User();

		if (input.isRegistration()) {
			user.setName(input.getName());
			user.setLastSeen(new Date());
			user.setStatus(User.UserStatus.ONLINE);
			user.setPasswordHash(input.getPasswordHash());
			persistenceManager.addUser(user);
		}else {
			// TODO: Implement error when user does not exist
			if (input.getPasswordHash() == user.getPasswordHash()) {
				user = persistenceManager.getUserByName(input.getName());
			} else {
				// TODO: Wrong Password Response
			}
		}
		ConnectionManager.getInstance().registerUser(socketId, user.getId());

		// Create contacts
		List<User>	users		= PersistenceManager.getPersistenceManager()
			.getContacts(user)
			.stream()
			.map(envoy.server.data.User::toCommonUser)
			.collect(Collectors.toList());
		Contacts	contacts	= new Contacts(user.getId(), users);

		// Complete handshake
		System.out.println("Sending user...");
		writeProxy.write(socketId, user.toCommonUser());
		System.out.println("Sending contacts...");
		writeProxy.write(socketId, contacts);
		System.out.println("Acquiring pending messages for the client...");
		List<Message> pendingMessages = PersistenceManager.getPersistenceManager().getUnreadMessages(user);
		for (Message msg : pendingMessages) {
			System.out.println("Sending message " + msg.toString());
			writeProxy.write(socketId, msg);
			msg.setReceivedDate(new Date());
			msg.setStatus(MessageStatus.RECEIVED);
			PersistenceManager.getPersistenceManager().updateMessage(msg);
		}
	}
}
